/*
 * infosv
 * sept 2012
 * Marco Antognini
 */

#include "Application.hpp"
#include <iostream>
#include <cassert>
#include "Config.h"

namespace {
    Application* currentApp = nullptr; ///< Current application
}

Application::Application()
: mPaused(false)
{
    assert(currentApp == nullptr);
    currentApp = this;

    // Most of the work is done in init() or create()
}

Application::~Application()
{

    // Release textures
    for (auto& kv : mTextures) {
        delete kv.second;
        kv.second = nullptr;
    }
    mTextures.clear();

    currentApp = nullptr;
}

void Application::init(int argc, char** argv)
{
    assert(argc >= 1);

    // Get the launch command (e.g. ./build/application)
    mAppDirectory = argv[0];
    // and keep only the directory (i.e. until the last '/')
    std::string::size_type lastSlashPos = mAppDirectory.rfind('/');
    if (lastSlashPos == std::string::npos) {
        mAppDirectory = "./";
    } else {
        mAppDirectory = mAppDirectory.substr(0, lastSlashPos + 1);
    }

    // Load the font
    if (!mFont.loadFromFile(mAppDirectory+"../res/arial.ttf")) {
        std::cerr << "Couldn't load font" << std::endl;
    }

    // Notify subclass(es)
    onInit(argc, argv);
}

void Application::run()
{
    // Create the window
    create();

    // Use a clock to track time
    sf::Clock clk;

    // Main loop
    while (mWindow.isOpen()) {
        // Handle events
        sf::Event event;
        while (mWindow.pollEvent(event)) {
            handleEvent(event, mWindow);
        }

        sf::Time dt = clk.restart() * (float)2.0; // Always reset the clock !
        if (!mPaused) {
            onUpdate(dt);
        }
        
        // Draw stuff
        onDraw(mWindow);

        // Display everything on the window
        mWindow.display();
    }
}

sf::Font const& Application::getFont() const
{
    return mFont;
}

sf::Texture& Application::getTexture(std::string const& name)
{
    auto const it = mTextures.find(name);
    if (it != mTextures.end()) return *it->second;
    else {
        // The texture is not yet in memory so we load it now
        sf::Texture* newTexture = new sf::Texture;
        if (newTexture->loadFromFile(mAppDirectory+"res/" + name)) {
            // The texture was correctly loaded so we save it
            mTextures[name] = newTexture;
            // And return the texture
            return *newTexture;
        } else {
            // The file was not loaded correctly so we clean up the mess
            delete newTexture;
            newTexture = nullptr;
            // And return the default texture
            return mDefaultTexture;
        }
    }
}
void Application::onInit(int, char**)
{
    // By default nothing is done here
}

void Application::onEvent(sf::Event, sf::RenderWindow&)
{
    // By default nothing is done here
}

void Application::onUpdate(sf::Time)
{
    // By default nothing is done here
}

void Application::onDraw(sf::RenderTarget&)
{
    // By default nothing is done here
}

void Application::create()
{
    // TODO Change the window size/style whatever
    sf::VideoMode vm(WINDOW_WIDTH, WINDOW_HEIGHT);
    sf::ContextSettings contextSettings;
    contextSettings.antialiasingLevel = 8; // This might not work very fast on the VMs...
    mWindow.create(vm,"InfoSV Pong", sf::Style::Close, contextSettings);

    mWindow.setKeyRepeatEnabled(true);
}

void Application::handleEvent(sf::Event event, sf::RenderWindow& window)
{
    switch (event.type) {
        case sf::Event::Closed:
            window.close();
            break;

        case sf::Event::KeyPressed:
            switch (event.key.code) {
                case sf::Keyboard::Escape:
                    window.close();
                    break;
                case sf::Keyboard::Space:
                    mPaused = ! mPaused; // toggle pause
                    break;

                default:
                    onEvent(event, window);
                    break;
            }
            break;

        default:
            onEvent(event, window);
            break;
    }
}

Application& getApp()
{
    assert(currentApp != nullptr);

    return *currentApp;
}


sf::Font const& getAppFont()
{
    return getApp().getFont();
}

sf::Texture& getAppTexture(std::string const& name)
{
    return getApp().getTexture(name);
}
